@@ -22,6 +22,7 @@ import re
 import codecs
 import os
 import yaml
+from distutils.version import StrictVersion
 from libopensesame import metadata
 from libopensesame.exceptions import osexception
 from libopensesame.py3compat import *
@@ -50,12 +51,17 @@ class syntax(object):
 		# - Consists only of letters, numbers, or underscores, except for
 		# - The last character, which is an equals sign
 		self.re_cmd = re.compile(r'\A[_a-zA-Z]+[_a-zA-Z0-9]*=')
-		# A regular expression to match [variables] in text
-		self.re_txt = re.compile(r'(?<!\\)(\[[_a-zA-Z]+[_a-zA-Z0-9]*\])')
+		# A regular expression to match [variables] in text. The first part
+		# indictates that we don't match items preceded by a slash
+		# (\[variables]). This introduces a problem in that the slash itself can
+		# also be escaped. To deal with this we eat up any even number of
+		# slashes (\\\\)* so that only sequences preceded by an odd number of
+		# slashes are preserved.
+		self.re_txt = re.compile(r'(?<!\\)(\\\\)*(\[[_a-zA-Z]+[_a-zA-Z0-9]*\])')
 		# A regular expression to match inline Python statements, like so:
 		# [=10*10]
 		# [\[test\]]
-		self.re_txt_py = re.compile(r'(?<!\\)(\[=.*?[^\\]\])')
+		self.re_txt_py = re.compile(r'(?<!\\)(\\\\)*(\[=.*?[^\\]\])')
 		# Catch single equals signs
 		self.re_single_eq = re.compile(r'(?<![=!<>])(=)(?!=)')
 		# Catch 'never' and 'always'
@@ -90,6 +96,9 @@ class syntax(object):
 
 		try:
 			f = float(val)
+			# No flanking whitespace
+			if isinstance(val, basestring):
+				assert(val == val.strip())
 		except:
 			return safe_decode(val, errors=u'ignore')
 		if int(f) == f:
@@ -115,12 +124,20 @@ class syntax(object):
 
 		m = self.re_front_matter.match(s)
 		try:
-			d = yaml.load(m.group(u'info'))
+			d = safe_yaml_load(m.group(u'info'))
 			s = s[len(m.group(0)):]
 		except:
 			d = {}
+		# The API can be stored in a few ways:
+		# - Not at all, which was the case in <= v2.9
+		# - As a single value, in which the case the minor version becomes 0
+		# - As a float, which is seen as a major.minor version
 		if u'API' not in d:
-			d['API'] = 1
+			d['API'] = StrictVersion(u'1.0')
+		elif isinstance(d['API'], int):
+			d['API'] = StrictVersion(u'%d.0' % d['API'])
+		elif isinstance(d['API'], float):
+			d['API'] = StrictVersion(str(d['API']))
 		return d, s
 
 	def generate_front_matter(self):
@@ -138,11 +155,15 @@ class syntax(object):
 
 		self.experiment.front_matter.update({
 			'OpenSesame' : safe_str(metadata.__version__),
-			'API' : 2,
+			'API' : metadata.api,
 			'Platform' : os.name
 			})
+		front_matter = self.experiment.front_matter.copy()
+		front_matter[u'API'] = metadata.api.version[0] \
+			if metadata.api.version[1] == 0 \
+			else float(str(metadata.api))
 		return u'---\n%s---\n' % safe_decode(yaml.dump(
-			self.experiment.front_matter, default_flow_style=False,
+			front_matter, default_flow_style=False,
 			allow_unicode=True))
 
 	def split(self, s):
@@ -265,6 +286,10 @@ class syntax(object):
 			The evaluated string, or the input value for non-string input.
 		"""
 
+		def get_escape_sequence(m):
+			return u'' if m.group(1) is None \
+				else m.group(1)[:len(m.group(1))//2]
+
 		if not isinstance(txt, basestring):
 			return txt
 		txt = safe_decode(txt)
@@ -276,20 +301,23 @@ class syntax(object):
 			m = self.re_txt.search(txt)
 			if m is None:
 				break
-			val = var.get(m.group()[1:-1])
+			varname = m.group(2)[1:-1]
+			val = var.get(varname)
 			if round_float and isinstance(val, float):
 				val = float_template % val
 			else:
 				val = safe_decode(val)
-			txt = txt[:m.start(0)] + val + txt[m.end(0):]
+			txt = txt[:m.start(0)] + get_escape_sequence(m) + val \
+				+ txt[m.end(0):]
 		# Detect Python inlines [=10*10]
 		while True:
 			m = self.re_txt_py.search(txt)
 			if m is None:
 				break
-			py = self.unescape(m.group()[2:-1])
+			py = self.unescape(m.group(2)[2:-1])
 			val = self.experiment.python_workspace._eval(py)
-			txt = txt[:m.start(0)] + safe_decode(val) + txt[m.end(0):]
+			txt = txt[:m.start(0)] + get_escape_sequence(m) + safe_decode(val) \
+				+ txt[m.end(0):]
 		return self.unescape(txt)
 
 	def quotable_symbol(self, s):
@@ -462,11 +490,13 @@ class syntax(object):
 		s = safe_decode(s).replace(u'\\', u'\\\\')
 		try:
 			float(s)
+			assert(s == s.strip())  # No flanking whitespace
 		except:
 			if not s.replace(u'_', u'').isalnum():
-				s = u'"%s"' % s.replace(u'"', u'\\"')
+				return u'"%s"' % s.replace(u'"', u'\\"')
 		return s
 
+
 	def sanitize(self, s, strict=False, allow_vars=True):
 
 		"""
